# pylint: disable-all
# flake8: noqa
# type: ignore
"""Summary."""

from typing import Any, List

from falcon import Request, Response, abort

from src.http_invoker import HttpInvoker


class FalconHttpInvoker(HttpInvoker):

    """Summary
    """

    def __init__(self, invocable: Any):
        """Summary

        Args:
            invocable (Any): Description
        """
        HttpInvoker.__init__(self, invocable)

    def on_get(self, request: Request, response: Response) -> None:
        """Summary

        Args:
            request (Request): Description
            response (Response): Description
        """
        data_out: List[Any] = []
        data_in: List[Any] = request.params
        # data_in(f'route: {str(request.url_rule)}')
        try:
            self._invocable.invoke(data_in, data_out)
        except Exception as err:
            print(str(err))
            abort(400)

        response.body = str(data_out)



"""Summary.

Attributes:
    VERSION (str): Description

"""
VERSION = '0.3.24-alpha'


def get_version() -> str:
    """Summary.

    Returns:
        str: Description

    """
    return VERSION


if __name__ == '__main__':
    print(get_version())



"""Summary."""
import importlib.util
import os
import re
import sys
from importlib.abc import Loader
from importlib.machinery import ModuleSpec
from types import ModuleType
from typing import Callable, Match, Optional

from src.payload import Payload
from src.types import TYPE_RETURN


class FunctionInvocable:
    """Summary."""

    def __init__(self, reference: str) -> None:
        """Summary.

        Args:
            reference (str): Description

        """
        self._func: Optional[Callable[..., TYPE_RETURN]] = None  # type: ignore
        self._reference: str = reference
        self.inject()

    @property
    def func(self) -> Optional[Callable[..., TYPE_RETURN]]:  # type: ignore
        """Summary.

        Returns:
            Optional[Callable[..., TYPE_RETURN]]: Description

        """
        return self._func

    @func.setter
    def func(self, arg: Callable[..., TYPE_RETURN]) -> None:  # type: ignore
        """Summary.

        Args:
            arg (Callable[..., TYPE_RETURN]): Description

        """
        self._func = arg

    def invoke(self, data_out: Payload, data_in: Payload) -> None:
        """Summary.

        Args:
            data_out (Payload): Description
            data_in (Payload): Description

        Raises:
            Exception: Description

        """
        result: TYPE_RETURN = None
        if not self._func:
            raise Exception('Cannot executeNo injected function')
        try:
            result = self._func(*data_in.list())
        except Exception as err:
            raise Exception(f'Referenced function {self._reference} threw an exception: {str(err)}')

        data_out.set('result', result)

    def inject(self) -> None:
        """Summary.

        Raises:
            Exception: Description

        """
        # [path/to/file/]<file>.<extension>[:[class.]method]]
        pattern: str = r'^(.*\/)?([^\.\/]+)\.([^\.]+):([^:]+\.)?([^:\.]+)$'  # (path/to/file/)(file).(extension):(method)
        matches: Optional[Match[str]] = re.match(pattern, self._reference)
        if not matches:
            raise Exception(f'Unable to inject invalid referenced function {self._reference}')

        path_to_source_file: str = matches.group(1)
        if not matches.group(1):
            path_to_source_file = os.getcwd()
        elif matches.group(1)[0] != '/':
            path_to_source_file = f'{os.getcwd()}/{matches.group(1)}'
        source_file_name: str = matches.group(2)
        source_file_extension: str = matches.group(3)
        sys.path.insert(0, path_to_source_file)

        spec: ModuleSpec = importlib.util.spec_from_file_location(source_file_name, f'{path_to_source_file}/{source_file_name}.{source_file_extension}')
        module: ModuleType = importlib.util.module_from_spec(spec)
        assert isinstance(spec.loader, Loader)  # see https://github.com/python/typeshed/issues/2793
        spec.loader.exec_module(module)

        scope: ModuleType = module
        if matches.group(4):
            class_name: str = matches.group(4)[:-1]
            scope = getattr(scope, class_name)

        method_name: str = matches.group(5)
        self._func = getattr(scope, method_name)



#!/usr/bin/env python
"""Summary

Attributes:
    VERSION (TYPE): Description
"""
import os
import subprocess
import sys
from distutils.core import setup

from setuptools.command.install import install

from src.version import get_version


def from_file(file_name):
    """print long description

    Args:
        file_name (TYPE): Description

    Returns:
        TYPE: Description
    """
    with open(file_name) as f:
        return f.read()


VERSION = get_version()


class VerifyVersionCommandx(install):
    """Custom command to verify that the git tag matches our version

    Attributes:
        description (str): Description
    """
    description = 'verify that the git tag matches our version'

    def run(self):
        """Summary
        """
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version

    Attributes:
        description (str): Description
    """
    description = 'verify that the git tag matches our version'

    def run(self):
        """Summary
        """
        tag = os.getenv('CIRCLE_TAG')
        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


setup(
    name='ergo',  # How you named your package folder (MyLib)
    packages=['src'],  # Chose the same as "name"
    version=VERSION,  # Start with a small number and increase it with every change you make
    license='MIT',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='Simple Microservice Development Framework',  # Give a short description about your library
    author='Matthew Hansen',  # Type in your name
    author_email='ergo@mattian.com',  # Type in your E-Mail
    url='https://github.com/mattian7741/ergo',  # Provide either the link to your github or to your website
    download_url=f'https://github.com/mattian7741/ergo/archive/v{VERSION}.tar.gz',  # github release url
    keywords=['execute', 'microservice', 'lambda'],  # Keywords that define your package best
    install_requires=[  # dependencies
        'Click',
        'flask',
        'falcon',
        'ansicolors',
        'click-default-group',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    long_description=from_file('README.md'),
    long_description_content_type='text/markdown',
    python_requires='>=3',
    entry_points={
        'console_scripts': [
            'ergo=src.ergo_click:main'
        ]
    },
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)
